//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   04 Mar 08  Brian Frank  Creation
//
package fan.sys;

import java.util.*;

/**
 * DirNamespace aliases a directory tree
 */
final class DirNamespace
  extends Namespace
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  DirNamespace(File dir)
  {
    if (!dir.isDir())  throw ArgErr.make("Not a dir: " + dir).val;
    if (!dir.exists()) throw ArgErr.make("Dir does not exist: " + dir).val;
    this.dir = dir;
  }

//////////////////////////////////////////////////////////////////////////
// Identity
//////////////////////////////////////////////////////////////////////////

  public String toStr() { return "DirNamespace " + dir; }

  public Type type() { return Sys.DirNamespaceType; }

//////////////////////////////////////////////////////////////////////////
// Namespace
//////////////////////////////////////////////////////////////////////////

  public Object get(Uri uri, boolean checked)
  {
    File f = dir.plus(uri.relTo(this.uri()), false);
    if (f.exists()) return f;
    if (!checked) return null;
    throw UnresolvedErr.make(uri).val;
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  private final File dir;

}
